/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : AirScheduleProblem.h                                     */
/*                                                                       */
/*       Date : April   4, 1997                                          */
/* Updated at : March  14, 1999                                          */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#ifndef _AIRSCHEDULEPROBLEM_H
#define _AIRSCHEDULEPROBLEM_H

class AirScheduleProblem;
class Column;

#include "Flight.h"
#include "City.h"
#include "Column.h"
#include "FlightGraph.h"
#include "EdgeGraph.h"
#include "TestDefinition.h"
#include "ListOfEdgeLists.h"

class AirScheduleProblem
{  public :
     AirScheduleProblem();
     ~AirScheduleProblem();
     AirScheduleProblem(const AirScheduleProblem&);

     void setDeadheadingCostRatio(float) ;
     void setCostPerMinute(float);
     void setCostPerMinuteUnderUtilization(float);
     void setCostPerOvernight(float);
     void setMinimumTime(float);
     void setLoadBalancingFactor(float);
     
     float getDeadheadingCostRatio() const;
     float getCostPerMinute() const;
     float getCostPerOvernight() const;
     float getMinimumTime() const;
     float getLoadBalancingFactor() const;
     float getTotalFlyingTime() const;

     unsigned getNumberOfFlights() const;

     unsigned getNumberOfCities() const;

     unsigned getNumberOfBaseCities() const;

     getLowerBound() ;

     FlightGraph createFlightGraph() const;
     FlightGraph getFlightGraph() const;

     City* getBaseCities() const;

     void addBaseCity(City);
     isCityExist(City);
     isBaseCity(City) const;

     getFlightIndexFromFlightNumber(unsigned FlightNumber);

     isFlightFromBaseCity(Flight *) const;
     isFlightToBaseCity(Flight *) const;

     isFlightFromBaseCity(int FlightIndex) const;
     isFlightToBaseCity(int FlightIndex) const;

     lookUp(char *);
     void addFlight(Flight);

     isRotationPossible(Column);

     Flight * getFlightWithIndex(int);
     City   * getCityWithIndex(int);
     EdgeGraph  *getEdgeGraph() { return myEdgeGraph; };
     void  setEdgeGraph(EdgeGraph *eGraph) { myEdgeGraph=eGraph; };

     void updateGraph();
     void updateFlightNumbers();
     void findBaseCities();

     AirScheduleProblem operator=(const AirScheduleProblem&);
     operator==(const AirScheduleProblem&);

     friend ostream& operator<<(ostream&,const AirScheduleProblem&);

     inline void setTestDefinition(TestDefinition _test) { test=_test;}

     inline float getMaxRotationPeriod()  
     { return test.getMaxRotationPeriod(); } ;

     inline float  getMaxDutyPeriod()  { return test.getMaxDutyPeriod(); } ;
     inline float  getAveDutyPeriod()  { return test.getAveDutyPeriod(); } ;
     inline float  getMaxFlyTime()     { return test.getMaxFlyTime(); } ;
     inline float  getMinPaidFlyTime() { return test.getMinPaidFlyTime(); } ;
     inline float  getMinSalary()      { return test.getMinSalary(); } ;

     inline float getBonusForExtraFlyMinute() 
     { return test.getBonusForExtraFlyMinute(); } ;

     inline float getMinRestTime()  { return test.getMinRestTime(); } ;
     inline float getMaxRestTime()  { return test.getMaxRestTime(); } ;
     inline float getHotelCost()    { return test.getHotelCost(); } ;
     inline float getPerDiemPerMinute() { return test.getPerDiemPerMinute(); } ;

     inline float getMinimumRestForDuty(float dtime) 
     { float ret = test.getMinRestTime();
       if(dtime>test.getAveDutyPeriod())
       {
	 ret += (dtime-test.getAveDutyPeriod())*(test.getMaxRestTime()-ret) / 
		(test.getMaxDutyPeriod()-test.getAveDutyPeriod());
       }
       return ret;
     } ;

     Date  getStartOfPlanHorizon() const;
     Date  getEndOfPlanHorizon() const;
     float getCostPerMinuteUtilization() const;

     void resetDifficulties();
     float getDifficulty(int indx);
     void incrementDifficulty(int indx);
     void buildPreSolutions();
     EdgeList* selectForNode(int fi, char *covered);
     void updatePreSolutions(EdgeList* eList );

   private:

     FlightGraph  myFlights;
        unsigned  numberOfFlights;
            City  *BaseCities;
        unsigned  bIndex;
           float  costPerMinute;
           float  costPerMinuteUnderUtilization;
           float  costPerOvernight;
           float  deadheadingCostRatio;
           float  loadBalanceFactor;
           float  minimumTime;
            City  *cities;
        unsigned  cIndex;
	  Flight  *FlightList;
       EdgeGraph  *myEdgeGraph;
  TestDefinition test;
	   float  *difficulty;
 ListOfEdgeLists  *sol;

}; // end of class AirScheduleProblem

#endif
